function [G, N, E] = dimSlepianCap( L, Theta, epsilon)
%
% [G, N, E] = dimSlepianCap( L, Theta, epsilon)
%
% Compute approximate dimension of spatio-spectral concentration problem
% when the region of concentration is a spherical cap with theta < Theta.
%
% @note The present formula is only a conjecture and in particular the
%       boundary term is not known exactly.
%
% @return G        Generalized Shannon number, approximate dimension
% @return N        Shannon number, number of eigenvalues very close to unity
% @return E        Number of eigenvalues in the region of the spectrum with
%                  exponential decay from 1 to 0.
% @param  L        Maximum Spherical Harmonics band
% @param  Theta    Maximum latitude of the spherical cap (in degrees)
% @param  epsilon  Epsilon threhold for the approximate dimension, that is
%                  G determines the number of eigenvalues larger than
%                  epsilon.

  % default input arguments
  defval( 'epsilon', 0.01);

  Theta = deg_to_rad(Theta);

  dim_sh = (L+1)^2;
  [A,dA] = areaSphericalCap( Theta);
  c = (dim_sh * A)/(4*pi);
  eps_factor = log10((1 - epsilon) / epsilon);
 
  dA_factor = log10( dim_sh * dA^2) / log10(2*pi) ;

  N = c + 1;
  E = log10(c*4*pi) * dA_factor * eps_factor;
  
  G = N + E;
 
end